app.controller('categoriesController', function($scope, $mdDialog, $mdToast, categoriesFactory){

	// read categories
	$scope.read = function(json_url){

		// use categories factory
		categoriesFactory.readCategories(json_url).then(function successCallback(response){
			$scope.categories = response.data.records;
			$scope.paging = response.data.paging;
		}, function errorCallback(response){
			$scope.showToast("Unable to read record.");
		});

	}

	// show 'create category form' in dialog box
	$scope.showCreateCategoryForm = function(event){

		$mdDialog.show({
			controller: DialogController,
			templateUrl: './app/categories/create_category.template.html',
			parent: angular.element(document.body),
			targetEvent: event,
			clickOutsideToClose: true,
			scope: $scope,
			preserveScope: true,
			fullscreen: true // Only for -xs, -sm breakpoints.
		});

	}

	// create new category
	$scope.createCategory = function(){

		categoriesFactory.createCategory($scope).then(function successCallback(response){

			// tell the user new category was created
			$scope.showToast(response.data.message);

			// refresh the list
			$scope.read('');

			// close dialog
			$scope.cancel();

			// remove form values
			$scope.clearCategoryForm();

		}, function errorCallback(response){
			$scope.showToast("Unable to create record.");
		});
	}

	// retrieve record to fill out the form
	$scope.readOneCategory = function(id){

		// get category to be edited
		categoriesFactory.readOneCategory(id).then(function successCallback(response){

			// put the values in form
			$scope.id = response.data.id;
			$scope.name = response.data.name;
			$scope.description = response.data.description;
			$scope.price = response.data.price;

			$mdDialog.show({
				controller: DialogController,
				templateUrl: './app/categories/read_one_category.template.html',
				parent: angular.element(document.body),
				clickOutsideToClose: true,
				scope: $scope,
				preserveScope: true,
				fullscreen: true
			}).then(
				function(){},

				// user clicked 'Cancel'
				function() {
					// clear modal content
					$scope.clearCategoryForm();
				}
			);

		}, function errorCallback(response){
			$scope.showToast("Unable to retrieve record.");
		});

	}

	// retrieve record to fill out the form
	$scope.showUpdateCategoryForm = function(id){

		// get category to be edited
		categoriesFactory.readOneCategory(id).then(function successCallback(response){

			// put the values in form
			$scope.id = response.data.id;
			$scope.name = response.data.name;
			$scope.description = response.data.description;
			$scope.price = response.data.price;

			$mdDialog.show({
				controller: DialogController,
				templateUrl: './app/categories/update_category.template.html',
				parent: angular.element(document.body),
				clickOutsideToClose: true,
				scope: $scope,
				preserveScope: true,
				fullscreen: true
			}).then(
				function(){},

				// user clicked 'Cancel'
				function() {
					// clear modal content
					$scope.clearCategoryForm();
				}
			);

		}, function errorCallback(response){
			$scope.showToast("Unable to retrieve record.");
		});

	}

	// update category record / save changes
	$scope.updateCategory = function(){

		categoriesFactory.updateCategory($scope).then(function successCallback(response){

			// tell the user category record was updated
			$scope.showToast(response.data.message);

			// refresh the category list
			$scope.read('');

			// close dialog
			$scope.cancel();

			// clear modal content
			$scope.clearCategoryForm();

		},
		function errorCallback(response) {
			$scope.showToast("Unable to update record.");
		});

	}

	// cofirm category deletion
	$scope.confirmDeleteCategory = function(event, id){

		// set id of record to delete
		$scope.id = id;

		// dialog settings
		var confirm = $mdDialog.confirm()
			.title('Are you sure?')
			.textContent('Category will be deleted.')
			.targetEvent(event)
			.ok('Yes')
			.cancel('No');

		// show dialog
		$mdDialog.show(confirm).then(
			// 'Yes' button
			function() {
				// if user clicked 'Yes', delete category record
				$scope.deleteCategory();
			},

			// 'No' button
			function() {
				// hide dialog
			}
		);
	}

	// delete category
	$scope.deleteCategory = function(){

		categoriesFactory.deleteCategory($scope.id).then(function successCallback(response){

			// tell the user category was deleted
			$scope.showToast(response.data.message);

			// refresh the list
			$scope.read('');

		}, function errorCallback(response){
			$scope.showToast("Unable to delete record.");
		});

	}

	// search categories
	$scope.searchCategories = function(json_url){

		// use categories factory
		categoriesFactory.searchCategories(json_url, $scope.category_search_keywords).then(function successCallback(response){
			$scope.categories = response.data.records;
			$scope.paging = response.data.paging;
		}, function errorCallback(response){
			$scope.showToast("Unable to read records.");
		});
	}

	// clear variable / form values
	$scope.clearCategoryForm = function(){
		$scope.id = "";
		$scope.name = "";
		$scope.description = "";
		$scope.price = "";
	}

	// show toast message
	$scope.showToast = function(message){
		$mdToast.show(
			$mdToast.simple()
				.textContent(message)
				.hideDelay(3000)
				.position("top right")
		);
	}

	// methods for dialog box
	function DialogController($scope, $mdDialog) {
		$scope.cancel = function() {
			$mdDialog.cancel();
		};
	}

});
